#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

"""配置"""

LOG_LEVEL = logging.INFO
DEFAULT_IMG_SUFFIX = '.png'

TIMESTAMP_WITH_FOLDER = True
URL = 'http://huaban.com/'
IMG_SAVE_PATH = 'C:/page_screenshot'
BASE_HEIGHT = 900
SCALE_HEIGHT = 1.25
TIME_MS_SCROLL_FAST = 600
TIME_MS_SCROLL_SLOW = 900
CHROME_DRIVER_PATH = 'C:/opt/exe/chromedriver.exe'
INIT_WINDOW_POSITION_X = 800
INIT_WINDOW_POSITION_Y = 0
JS_FILE_NAME = 'return nickname + "-" + msg_title'
JS_MD5_URL = 'return md5(window.location.href)'
INIT_HEIGHT = 500  # 响应式网页自行调整高度哦

# 微信公众号宽屏
OUTER_PC_WIDTH = 1273

# 手机屏
OUTER_PHONE_WIDTH = 643

"""配置 end"""

"""javascript"""
JS_MAX_HEIGHT = """
return
Math.max(document.body.scrollHeight, document.body.offsetHeight
, document.documentElement.clientHeight, document.documentElement.scrollHeight, document.documentElement.offsetHeight);
""".replace("\n", " ")

JS_MAX_WIDTH = """
return
Math.min(document.body.scrollWidth, document.body.offsetWidth
, document.documentElement.clientWidth, document.documentElement.scrollWidth, document.documentElement.offsetWidth);
""".replace("\n", " ")

JS_WINDOW_HEIGHT = 'return document.documentElement.clientHeight'

JS_SCROLL = 'setTimeout(function(){ window.scrollTo(0, %s); }, %s);'

"""javascript end"""


def init():
    """
    初始化

    :return: void
    """
    logging.basicConfig(level=LOG_LEVEL, format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s', datefmt='[%Y-%M-%d %H:%M:%S]')


"""run"""
init()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@project: page screenshot
@author: xiaohong
@time: 2019-06-07
@feature: a complete capture from a web page
"""

import time
import os
from PIL import Image
from selenium import webdriver

from page_screenshot.model.model import *


def make_unique_folder(path):
    """
    创建时间戳文件夹

    :param path: 文件路径
    :return: 文件夹路径
    """
    t = ''
    # if TIMESTAMP_WITH_FOLDER:
    #     t = '/' + str(int(time.time()))
    folder = path + t
    if not os.path.isdir(folder):
        os.makedirs(folder)
    return folder


class PageScreenshot:
    def __init__(self):
        """
        init
        """
        self.model = None
        self.driver = self.take_driver()

    def print_w_h(self):
        """
        print

        :return: void
        """
        width = self.driver.execute_script(JS_MAX_WIDTH)
        height = self.driver.execute_script(JS_MAX_HEIGHT)
        driver_width = self.driver.get_window_size()['width']
        driver_height = self.driver.get_window_size()['height']
        window_height = self.driver.execute_script(JS_WINDOW_HEIGHT)

        logging.info('外宽(set_window_size) %s, %s; 外宽截图 %s, %s; 内宽(JS_MAX_WIDTH/HEIGHT) %s %s; 内宽截图 %s, %s'
                     % (driver_width, driver_height,
                        driver_width * self.model.scale_window, driver_height * self.model.scale_window,
                        width, height,
                        width * self.model.scale_window, window_height))

    def calc_right_step_scroll(self):
        """
        计算滚动大小

        :return: 滚动次数，滚动高度
        """
        times_shot = 1
        inner_height = height = float(self.driver.execute_script(JS_MAX_HEIGHT))

        # # 没有超出基本高度
        # if inner_height <= self.model.base_height:
        #     return times_shot, window_height

        # 计算不超出基本高度的截图高度
        while True:
            if inner_height <= self.model.base_height:
                break
            times_shot = times_shot + 1
            inner_height = height / times_shot
        # 保持宽度不变，修改高度为内高的变率倍
        driver_height = self.driver.get_window_size()['height']
        window_height = self.driver.execute_script(JS_WINDOW_HEIGHT)
        if 0 == window_height:
            window_height = 1
        self.driver.set_window_size(self.driver.get_window_size()['width'], inner_height * driver_height / window_height)

        # 获取截图高度
        logging.info('times_shot : %s' % times_shot)
        self.print_w_h()
        return times_shot, self.driver.execute_script(JS_WINDOW_HEIGHT)

    def scroll_to_capture(self):
        """
        滚动并截图

        :return: 截图次数
        """
        self.model.img_save_path = make_unique_folder(self.model.img_save_path)
        times_shot, step_scroll = self.calc_right_step_scroll()

        # 快速滚动一遍
        for i in range(times_shot):
            self.driver.execute_script(JS_SCROLL % (i * step_scroll, self.model.time_ms_scroll_fast))
            time.sleep(self.model.time_ms_scroll_fast * 0.002)

            # 滚动到顶部
            self.driver.execute_script(JS_SCROLL % (0, self.model.time_ms_scroll_fast))
        times_shot, step_scroll = self.calc_right_step_scroll()

        # 滚动两遍，确保已经加载完毕
        for i in range(times_shot):
            self.driver.execute_script(JS_SCROLL % (i * step_scroll, self.model.time_ms_scroll_slow))
            time.sleep(self.model.time_ms_scroll_slow * 0.002)

            self.driver.save_screenshot(self.model.img_save_path + '/' + str(i) + DEFAULT_IMG_SUFFIX)
        time.sleep(3)
        return times_shot

    @staticmethod
    def take_driver():
        """
        生成driver

        :return: WebDriver
        """
        options = webdriver.ChromeOptions()
        # options.binary_location = 'C:/Users/xh/AppData/Roaming/360se6/Application/360se.exe'  # 修改成360浏览器
        # options.add_argument('--headless')  # 无法加载异步请求的图片，即滚动事件不会进行图片加载
        options.add_argument('--dns-prefetch-disable')
        options.add_argument('--no-referrers')
        options.add_argument('--disable-gpu')
        options.add_argument('--disable-audio')
        options.add_argument('--no-sandbox')
        options.add_argument('--ignore-certificate-errors')
        options.add_argument('--allow-insecure-localhost')
        # options.add_argument('--switcher-file=C:/Users/xh/Documents/PyCharmProjects/page_screenshot/switch_360_chrome.txt')  # 修改成360浏览器
        options.add_argument('disable-infobars')

        driver = webdriver.Chrome(options=options, executable_path=CHROME_DRIVER_PATH)
        return driver

    def take_url(self):
        self.driver.get(url=self.model.url)
        time.sleep(.5)
        self.driver.set_window_position(self.model.init_window_position_x, self.model.init_window_position_y)

        # self.driver.execute_script("document.body.style.zoom='125%'")  # 修改成360浏览器
        self.driver.execute_script('document.body.parentNode.style.overflowX = "hidden";')  # 隐藏滚动条
        self.driver.execute_script('document.body.parentNode.style.overflowY = "hidden";')  # 隐藏滚动条

        self.driver.set_window_size(self.model.outer_width / self.model.scale_window, self.model.init_height)

    def check(self):
        template_file_name = self.model.default_img_save_path + '/%s' + DEFAULT_IMG_SUFFIX
        return os.path.exists(template_file_name % self.model.file_name)

    def capture(self, page_screenshot_model: PageScreenshotModel):
        """
        生成网页截图文件

        :param page_screenshot_model: PageScreenshotModel
        :return: void
        """
        self.model = page_screenshot_model
        self.take_url()

        # noinspection PyBroadException
        try:
            # 执行指定的js脚本，建立文件名
            self.model.file_name = self.driver.execute_script(self.model.js_file_name)
        except Exception:
            # noinspection PyBroadException
            try:
                # 使用url的md5值作为文件名
                self.model.file_name = self.driver.execute_script(JS_MD5_URL)
            except Exception:
                # 使用时间戳作为文件名
                self.model.file_name = str(int(time.time()))

        # 过滤不合符文件名字符
        self.model.file_name = self.filter_file_name(self.model.file_name)

        # 检查文件是否存在
        if self.check():
            logging.info('图片已存在 %s' % self.model.file_name)
            return

        # capture now
        logging.info('开始捕获截图的url %s' % page_screenshot_model.url)
        self.model.count_image = self.scroll_to_capture()
        logging.info('merge image ...')
        self.merge()
        logging.info('remove_temp_file ...')
        self.remove_temp_file()

    def close(self):
        """
        close

        :return: void
        """
        self.driver.quit()

    def remove_temp_file(self):
        """
        删除每次滚动截图文件

        :return: void
        """
        template_file_name = self.model.img_save_path + '/%s' + DEFAULT_IMG_SUFFIX
        for i in range(self.model.count_image):
            # noinspection PyBroadException
            try:
                os.remove(template_file_name % i)
            except Exception:
                print('no image file, go on')

    @staticmethod
    def filter_file_name(file_name: str):
        """
        过滤非法文件名称的字符
        :param file_name: 文件名
        :return: 正确的文件名
        """
        illegal_char = ['\\', '/', ':', '*', '?', '"', '<', '>', '|']
        for i in illegal_char:
            file_name = file_name.replace(i, '')
        return file_name

    def merge(self):
        """
        拼接滚动截图文件

        :return: void
        """
        template_file_name = self.model.img_save_path + '/%s' + DEFAULT_IMG_SUFFIX

        f = Image.open(template_file_name % 0)
        w, h = f.size
        merge_image = Image.new('RGBA', (w, h * self.model.count_image))
        for i in range(self.model.count_image):
            merge_image.paste(Image.open(template_file_name % i), (0, i * h))

        # save to one image file
        # noinspection PyBroadException
        try:
            merge_image.save(template_file_name % self.model.file_name)
        except Exception:
            merge_image.save(template_file_name % str(int(time.time())))
            logging.info('文件名称不合符 %s' % self.model.url)
        time.sleep(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from page_screenshot.config.setting import *


class PageScreenshotModel:
    @staticmethod
    def build(url, img_save_path=IMG_SAVE_PATH, js_file_name=JS_FILE_NAME, outer_width=OUTER_PC_WIDTH, init_height=INIT_HEIGHT):
        if str(url).strip() == '':
            raise SystemExit('url 不能为空')
        page_screenshot_model = PageScreenshotModel()
        page_screenshot_model.url = url
        page_screenshot_model.img_save_path = img_save_path
        page_screenshot_model.default_img_save_path = img_save_path
        page_screenshot_model.js_file_name = js_file_name
        page_screenshot_model.outer_width = outer_width
        page_screenshot_model.init_height = init_height
        return page_screenshot_model

    # timestamp_with_folder = TIMESTAMP_WITH_FOLDER
    url = URL
    img_save_path = IMG_SAVE_PATH
    default_img_save_path = IMG_SAVE_PATH

    base_height = BASE_HEIGHT
    scale_window = SCALE_HEIGHT

    time_ms_scroll_fast = TIME_MS_SCROLL_FAST
    time_ms_scroll_slow = TIME_MS_SCROLL_SLOW
    chrome_driver_path = CHROME_DRIVER_PATH

    init_window_position_x = INIT_WINDOW_POSITION_X
    init_window_position_y = INIT_WINDOW_POSITION_Y
    js_file_name = JS_FILE_NAME

    outer_width = OUTER_PC_WIDTH
    init_height = INIT_HEIGHT

    # 中间变量
    count_image = None
    file_name = None  # 无后缀

    def set_outer_width(self, outer_width):
        self.outer_width = outer_width

    def add_to_js_file_name(self, js_file_name):
        self.js_file_name = self.js_file_name + ' + "-%s"' % js_file_name

    def set_js_file_name(self, js_file_name):
        self.js_file_name = js_file_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@project: page screenshot
@author: xiaohong
@time: 2019-06-07
@feature: a complete capture from a web page
"""

import io
import os
import time
import shutil
import logging
import threading
from page_screenshot.model.model import PageScreenshotModel, OUTER_PC_WIDTH, OUTER_PHONE_WIDTH, TIMESTAMP_WITH_FOLDER
from page_screenshot.index import PageScreenshot
import random


def make_unique_folder(path, timestamp_with_folder=TIMESTAMP_WITH_FOLDER):
    """
    创建时间戳文件夹

    :param path: 文件路径
    :param timestamp_with_folder: 文件夹随机码
    :return: 文件夹路径
    """
    t = ''
    if timestamp_with_folder:
        t = '/' + str(int(time.time())) + str(random.randint(0, 1000000))
    folder = path + t
    if not os.path.isdir(folder):
        os.makedirs(folder)
    return folder


def run_thread(url_list=None, responsive_height=None):
    if url_list is None or 0 == len(url_list):
        return
    page_screenshot = PageScreenshot()

    # 建立唯一的目录
    url = url_list[0]
    page_screenshot_model = screenshot_for_pc(url=url, responsive_height=responsive_height)
    page_screenshot_model.img_save_path = make_unique_folder(page_screenshot_model.default_img_save_path)

    # js file name
    default_js_file_name = page_screenshot_model.js_file_name
    for url in url_list:
        # 修改默认目录，重复利用model对象
        # pc
        page_screenshot_model = screenshot_for_pc(url=url, page_screenshot_model=page_screenshot_model, responsive_height=responsive_height)
        page_screenshot.capture(page_screenshot_model)

        time.sleep(1)

        # phone
        page_screenshot_model = screenshot_for_phone(url=url, page_screenshot_model=page_screenshot_model, responsive_height=responsive_height)
        page_screenshot.capture(page_screenshot_model)

        # revert
        page_screenshot_model.js_file_name = default_js_file_name
    # close
    page_screenshot.close()

    logging.info('move file ...')
    # move file to default path
    for f in os.listdir(page_screenshot_model.img_save_path):
        shutil.move(page_screenshot_model.img_save_path + '/' + f, page_screenshot_model.default_img_save_path + '/' + f)

    # remove folder
    # noinspection PyBroadException
    try:
        shutil.rmtree(page_screenshot_model.img_save_path)
    except Exception as e:
        logging.info('remove folder error %s' % e)


def run_for_url_list(file_path, responsive_height=None, count_thread=10):
    with io.open(file_path, encoding='utf-8') as f:
        url_list = f.read()
    url_list = url_list.split('\n')
    if len(url_list) == 0:
        logging.info('file have not a url')

    if count_thread <= 0:
        count_thread = 10
    if len(url_list) < count_thread:
        count_thread = len(url_list)
    count_url_per_thread = len(url_list) / count_thread
    threads = []
    count_url_per_thread = int(count_url_per_thread)
    last = count_thread - 1

    for i in range(count_thread):
        start = i * count_url_per_thread
        if i == last:
            threads.append(threading.Thread(target=run_thread, args=(url_list[start:], responsive_height)))
        else:
            threads.append(threading.Thread(target=run_thread, args=(url_list[start: start + count_url_per_thread], responsive_height)))

    for th in threads:
        th.start()
    for th in threads:
        th.join()


def run(url, responsive_height=None):
    """
    run

    :param responsive_height: responsive_height
    :param url: url
    :return: void
    """
    page_screenshot = PageScreenshot()
    screenshot_for_pc_and_phone(url, page_screenshot, responsive_height)
    page_screenshot.close()


def screenshot_for_pc_and_phone(url, page_screenshot, responsive_height=None):
    """
    pc and phone

    :param responsive_height: responsive_height
    :param url: url
    :param page_screenshot: page_screenshot
    :return: void
    """
    page_screenshot_model = screenshot_for_pc(url=url, responsive_height=responsive_height)
    page_screenshot.capture(page_screenshot_model)

    page_screenshot_model = screenshot_for_phone(page_screenshot_model=page_screenshot_model, responsive_height=responsive_height)
    page_screenshot.capture(page_screenshot_model)


def screenshot_for_pc(url=None, page_screenshot_model=None, responsive_height=None):
    """
    pc

    :param responsive_height: responsive_height
    :param url: url
    :param page_screenshot_model: model
    :return: page_screenshot_model
    """
    if page_screenshot_model is None:
        page_screenshot_model = PageScreenshotModel.build(url)
    if url is not None:
        page_screenshot_model.url = url
    if responsive_height is not None:
        page_screenshot_model.init_height = responsive_height
    page_screenshot_model.set_outer_width(OUTER_PC_WIDTH)
    return page_screenshot_model


def screenshot_for_phone(url=None, page_screenshot_model=None, responsive_height=None):
    """
    phone

    :param responsive_height: responsive_height
    :param url: url
    :param page_screenshot_model: model
    :return: page_screenshot_model
    """
    if page_screenshot_model is None:
        page_screenshot_model = PageScreenshotModel.build(url)
    if url is not None:
        page_screenshot_model.url = url
    if responsive_height is not None:
        page_screenshot_model.init_height = responsive_height
    page_screenshot_model.set_outer_width(OUTER_PHONE_WIDTH)
    page_screenshot_model.add_to_js_file_name('phone')
    return page_screenshot_model



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@project: take screenshot on a web page
@author: xiaohong
@time: 2019-06-06
@feature: a complete capture from a web page/have a free choose in web page width/
"""

import setuptools
import io

with io.open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name="page_screenshot",
    version="0.0.6",

    author="xiaohong2019",
    author_email="2229009854@qq.com",

    description="a complete capture from a web page",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/xiaohong2019/page_screenshot",
    packages=setuptools.find_packages(),
    install_requires=[
        'selenium==3.11.0',
        'Pillow',
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Microsoft :: Windows",
    ],
)
